package textanalyzer.model;

import java.net.*;
import java.io.*;

public 
class      ResourceManager
implements Runnable {

    public static final int 
        N = 1000; // buffer size
    
    public static final String 
        FILE  = "file://",
        FILE1 = "file:///",
        HTTP  = "http://",
        FTP   = "ftp://";
   
    private static final String
        _empty = "",
        _malformedURL  = " :    ",
        _cannotConnect = " :   ",
        _cannotLoad    = " :   ",
        _kbyte         = "k",
        _trying        = " ",
        _connecting    = " ",
        _loading       = ":",
        _interrupted   = "",
        _displaying    = " ",
        _ready         = "";

    private AppUI ui;
    private AnalyzerModel analyzer;
    
    public
    ResourceManager( AppModel app ) {
        ui    = app.getUI();
        data  = new StringBuffer();
    }
    
    public 
    void addAnalyzer( AnalyzerModel analyzer ) {
        this.analyzer = analyzer;
    }
    
    private String location;
    private URL url;
    private URLConnection connection;
    private InputStream stream;
    private StringBuffer data;

    public 
    StringBuffer getData() {
        return data;
    }
    
    public 
    String getDataString() {
        return data.toString();
    }

    private 
    void   load() {
        
        // try to correct URL
        while( location.startsWith(" ") ) {
            location = location.substring( 1, location.length() );
        }
        if ( location.startsWith(FILE) && 
             !location.startsWith(FILE1) ) {
            location = location.substring( 7, location.length() );
            location = FILE1 + location;
        }
        ui.showLocation( location );
        
        String suppose;
        if ( location.startsWith(FILE) ||
             location.startsWith(HTTP) || 
             location.startsWith(FTP) ) {
            if ( !goodURL( location, true ) ) {
                return;
            }
        } else {

            // suppose file
            if (location.startsWith("/")) 
                suppose = FILE + location;
            else 
                suppose = FILE1 + location;
            if ( goodURL( suppose, false ) ) {
                location = suppose;
            } else {

                // no such file, suppose http
                suppose = HTTP + location;
                if ( goodURL( suppose, false ) ) {
                    location = suppose;
                } else {
    
                    // no such http, suppose ftp
                    suppose = FTP + location;
                    if ( goodURL( suppose, false ) ) {
                        location = suppose;
                    } else {
                        // giving up
                        ui.showLocation(location);
                        ui.showMessage( location + _malformedURL );
                        return;
                    }
                }
            }
        }
        
        // here we get good URL and try to open InputStream
        try {
            url = new URL( location );
            ui.showStatus(_connecting);
            connection = url.openConnection();
            ui.showStatus(_loading);
            stream = connection.getInputStream();
        } catch ( MalformedURLException e ) {
            ui.showStatus(_empty);
            ui.showMessage( location + _malformedURL );
            return;
        } catch ( IOException e ) {
            ui.showStatus(_empty);
            ui.showMessage( location + _cannotConnect );
            return;
        } catch ( Exception e ) {
            ui.showStatus(_empty);
            ui.showMessage( location + e.toString() );
            return;
        }
        
        // here we get opened InputStream, and try to read it
        int c;
        int n = 0;
        byte buffer[] = new byte[N]; 
        String enc = connection.getContentEncoding();
        String text;
        data.setLength(0);
        ui.showText();
        try {
            while ( ( c = stream.read(buffer) ) != -1 ) {
                n += c;
                ui.showStatus( _loading + n/1000 + _kbyte );
                if ( enc != null )
                    text = new String( buffer, 0, c, enc );
                else
                    text = new String( buffer, 0, c );
                data.append( text );
            }
            stream.close();
        } catch ( IOException e ) {
            ui.showStatus(_empty);
            ui.showMessage( location + _cannotLoad );
            return;
        } catch ( Exception e ) {
            ui.showStatus(_empty);
            ui.showMessage( location + e.toString() );
            return;
        }
        ui.showStatus(_displaying);
        ui.showText();
        analyzer.newText( data.toString() );
        ui.showStatus(_ready);
    }
    
    
    // check URL
    private 
    boolean goodURL( String location, boolean show ) {
        try {
            ui.showLocation( location );
            ui.showStatus( _trying );
            url = new URL( location );
            connection = url.openConnection();
            stream = connection.getInputStream();
            stream.close();
        } catch ( MalformedURLException e ) {
            ui.showStatus(_empty);
            if ( show )
                ui.showMessage( location + _malformedURL );
            return false;
        } catch ( IOException e ) {
            ui.showStatus(_empty);
            if ( show )
                ui.showMessage( location + _cannotLoad );
            return false;
        } catch ( Exception e ) {
            ui.showStatus(_empty);
            ui.showMessage( e.toString() );
            return false;
        }
        return true;
    }
    
    
    
    // thread implementation

    private Thread resourceThread;
    
    // load resource
    public 
    void   load( String location ) {
        this.location = location;
        if ( resourceThread != null ) stop();    
        resourceThread = new Thread( this, "ResourceManager" );
        resourceThread.start();
    }

    public void run() {
		//Thread.currentThread().setPriority( Thread.MIN_PRIORITY );
        load(); // call private method
        resourceThread = null;
    }
	
    // stop loading
    public void stop() {
        if ( resourceThread != null ) {
            ui.showStatus(_interrupted);
            resourceThread.stop();
            resourceThread = null;
            if ( stream != null ) 
                try{
                    stream.close();
                } catch ( IOException e ) {}
        }
    }
      
    
}
