package textanalyzer.model.knowledge;

import textanalyzer.model.*;
import java.util.Enumeration;
import java.io.IOException;

public 
class      InteractiveTextAnalyzer 
extends    TextParser
implements AnalyzerModel {
    
    private AnalyzerUI ui;

    private boolean work;
    
    public InteractiveTextAnalyzer( KnowledgeAgent agent, ResourceManager mgr, AnalyzerUI ui ) {
        super( agent );
        this.ui = ui;
        mgr.addAnalyzer( this );
        maxCo = 1; 
        work = false;
    }
    
    public void startWork() {
        while ( true ) {
            //if ( work ) {
            //    parse();
            //    work = false;
            //}
            sleep();
        }
    }
    
    public void stopWork() {
    }
    
    public void newText( String text ) {
        this.text = text;
        ui.clear();
        saveDictionary("dictionary.dic");
        knowledge.clear();
        loadDictionary("dictionary.dic");
        
        for (int i=0; i<N; i++) top[i]=null;
        maxCo = 1;
        
        parse();
        sort();
        WO = null;
        updateUI(1);
    }
    
    public void markStopWord(String word) {
        Word w = knowledge.recognize( word ); 
        if (word != null ) w.stopword = true;
    }
    
    public void loadDictionary(String filename){
        knowledge.load(filename);
    }
    
    public void saveDictionary(String filename) {
        try{
            knowledge.save(filename);
        } catch (IOException e) {}
    }

    //public void appendText( String text ) {
    //    this.text += text;
    //    work = true;
    //}

    //public void endText() {
    //    this.text += ".";
    //    work = true;
    //}
    
    
    // sort
    private static final int n = 30, N = n*10;
    private Word top[] = new Word[N];
    private Word W, tmp;
    private int co, co2, maxCo; 

    public void sort() {
        //start fill
        W = new Word("zzz");
        for(int i=0;i<N;i++) {
            top[i] = W;
        }
        // bubble-like top sort
        while ( WO != null ) {
            W = WO.getWord();
            int i,j;
            for (i=N-1; i>=0; i--) {
                if ( W.value().equals(top[i].value()) ) break;
                co = W.getCorrelation(W).get();
                co2 = top[i].getCorrelation(top[i]).get();
                if (co > maxCo) maxCo = co;
                if (co2<co) { // ? <= 
                    // shift bottom
                    for(j=N-1; j>i; j--) top[j]=top[j-1];
                    top[i] = W; 
                    // bubble to upper
                    for(j=i; j>0; j--) {
                        co = top[j-1].getCorrelation(top[j-1]).get();
                        co2 = top[j].getCorrelation(top[j]).get();
                        if (co2>co) { // ? >=
                            tmp = top[j-1];
                            top[j-1] = top[j];
                            top[j] = tmp;
                        }
                    }    
                    break;
                }
                
            }    
            WO = WO.getPrevious();
        }
    }
    
    // update UI
    public void updateUI(double scale) {     
        int i, j, weight;
        int start = 0, end = N;
        for (i=start; i<end; i++) {
            if (top[i]==null) continue;
            ui.updateEntity( top[i].value(),  
                             top[i].getCorrelation(top[i]).get() / (double)maxCo );
            for (j=start; j<end; j++) {
                if (top[j]==null) continue;
                weight = top[i].getCorrelation(top[j]).get();
                if (weight != 0)
                ui.updateRelation( top[i].value(), 
                                   top[j].value(), 
                                   weight /(double)maxCo );
            }
        }
    } 
        
    

}
