package textanalyzer.model.knowledge;

public
abstract
class      KnowledgeWorker 
implements Runnable {
    
    private static final long SLEEP_TIME = 500; // ms

    private   KnowledgeAgent agent;
    private   Thread         workerThread;
    protected WordSystem     knowledge;
    
    public 
    KnowledgeWorker( KnowledgeAgent agent ) {
        this.agent  = agent;
        knowledge   = agent.getKnowledge();
    }
        
    
    public void run() {
		//Thread.currentThread().setPriority( Thread.MIN_PRIORITY );
        startWork();
        agent.finishedWorker( this );
        workerThread = null;
    }
	
    public void start() {
        if ( workerThread == null )
            workerThread = new Thread( this, "KnowledgeWorker" );
        workerThread.start();
    }

    public void stop() {
        stopWork();
        workerThread.stop();
        workerThread = null;
        agent.finishedWorker( this );
    }
    
    public void suspend() {
        workerThread.suspend();
    }

    public void resume() {
        workerThread.resume();
    }

    public void sleep() {
        try {
            workerThread.sleep( SLEEP_TIME );
        } catch ( InterruptedException e ) {
            stop();
        }
    }
    
    public abstract void startWork();
    
    public abstract void stopWork();
    
}
