package textanalyzer.model.knowledge;

import java.util.Hashtable;
import java.util.Enumeration;
import java.io.*;

public 
class WordSystem {
    
    private Hashtable Words;
    private static int   DEFAULT_SIZE = 1000;
    private static float DEFAULT_LOAD = (float)0.8;
    
    public 
    WordSystem() {
        Words = new Hashtable( DEFAULT_SIZE, DEFAULT_LOAD );
    }
    
    private
    boolean contains( String key ) {
        return ( Words.get( key ) != null );
    }

    private
    Enumeration enumerate() {
        return Words.keys();
    }

    protected 
    Word recognize ( String sample ) {
        Word found = (Word)Words.get( sample );
        if ( found == null ) {
            // Word unknown
            found = new Word(sample);    // create empty Word
            Words.put( sample, found );  // remember it
        }
        return found;
    }

    protected
    Word teach ( String sample, Word example ) {
        Word known = (Word)Words.get( sample );
        if ( known == null ) {
            // unknown Word, remember
            Words.put( sample, example ); 
        } else {
            // known Word, teach
            //Word S_improved = new Word();
            //if ( S instanceof WordWord ) {
            //}
        }
        return known;
    }
    
    protected
    void teach( WordSystem subject ) {
        Enumeration subjectSamples = subject.enumerate();
        String subjectSample;
        while ( subjectSamples.hasMoreElements() ) {
            subjectSample = (String)(subjectSamples.nextElement());
            teach( subjectSample, subject.recognize( subjectSample ) );
        }
    }
    
    protected void clear() {
        Words.clear();
    }
    
    protected void load( String filename ) {//throws IOException {
        FileReader reader;
        try {
            reader = new FileReader(filename);
        } catch (IOException e) {
            System.err.println( e.toString() );
            return;
        }
        StreamTokenizer st = new StreamTokenizer(reader);
        Word W;
        
        try {
        while ( st.nextToken() != StreamTokenizer.TT_EOF ) {
            if ( st.ttype==StreamTokenizer.TT_WORD ) {
                W = recognize( st.sval );
                W.stopword = true;  // reads stopwords 
            }
        } 
        reader.close();  
        } catch (IOException e){}
    }

    protected
    void save( String filename ) throws IOException {
        // save stopwords
        FileWriter writer;
        Word W;
        writer = new FileWriter(filename);
        Enumeration enumWords = Words.elements();
        while ( enumWords.hasMoreElements() ) {
            W = (Word)(enumWords.nextElement());
            String s = W.value() + "\n\r";
            if (W.stopword) writer.write(s);
        }
        writer.close();
    }

}
