package textanalyzer.ui;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import textanalyzer.*;
import textanalyzer.model.*;
import textanalyzer.ui.view.graph.*;
import java.io.IOException;


public 
class      MainApplet 
extends    Applet  
implements AppUI, KeyListener {
    
    public    AppModel model;
    public    AnalyzerModel analyzerModel;
    public    AppModel.Properties properties;
    protected Frame frame; 

    private MainControlPanel controlPanel;
    private Panel      centerPanel;
    private GraphView  graphView;
    private TextArea   textView;
    private boolean    displayText;
			
    public MainApplet() {
        model   = new TextAnalyzer( this );
        properties = model.getProperties();
        _appname = properties.Name;
        displayText = false;
    }
    
    public static void main (String[] args) {
        MainFrame frame = new MainFrame( new MainApplet() );
    }
    
    public void init() {

        controlPanel = new MainControlPanel( this );
        centerPanel = new Panel();
		graphView = new GraphView( this );
        textView = new TextArea();
        textView.setEditable( false );
		textView.setBackground( properties.UIBgColor );
		centerPanel.setLayout( new GridLayout( 1, 2) );
		centerPanel.add( graphView );
        
        setFont( properties.UIFont );
        setBackground( properties.UIColor );
        setLayout( new BorderLayout() );
        add( controlPanel, BorderLayout.NORTH );
		add( centerPanel, BorderLayout.CENTER );
        
        addKeyListener( this );
        textView.addKeyListener( this );
        graphView.addKeyListener( this );
        controlPanel.addKeyListener( this );
        
        // !!!
        analyzerModel = model.addInteractiveAnalysis( graphView.getModel() );
        graphView.getModel().setModel( analyzerModel );

        analyzerModel.loadDictionary("dictionary.dic");
	}

    public void destroy() {
        remove(graphView);
        remove(textView);
        remove(controlPanel);
        
        analyzerModel.saveDictionary("dictionary.dic");
        
    }

    public void start() {
        graphView.start();
    }

    public void stop() {
        graphView.stop();
    }

    public String getAppletInfo() {
        return "Applet:  " + properties.Name +
             "\nVersion: " + properties.Version +
             "\nAuthor:  " + properties.Author;
    }
    
    private String 
        _appname = "",
        _has     = " [",
        _status  = "",
        _end     = "]",
        _s       = "";
    
    private
    void updateTitle() {
        if ( frame != null ) {
            if ( _status.length() == 0 ) _s = "";
            else _s = _has + _status + _end;
            frame.setTitle( _appname + _s );
        }
    }

    public 
    synchronized                    
    void showLocation( String location ) {
        controlPanel.setLocation( location );
    }

    public 
    synchronized                    
    void showStatus( String status ) {
        if ( frame == null ) {
            super.showStatus( status );
        } else {
            _status = status;
            updateTitle();
        }
    }

    public 
    synchronized                    
    void showMessage( String msg ) {
        MainMessage message = new MainMessage( this, msg );
        message.show();
    }
    

    public 
    boolean getDisplayText() {
        return displayText;
    }
    
    public 
    void toggleDisplayText() {
        if ( displayText ) {
            centerPanel.remove( textView );
            centerPanel.doLayout();
            displayText = false;
        } else {
            centerPanel.add( textView );
            centerPanel.doLayout();
            displayText = true;
            showText();
        }
    }

    public 
    synchronized                    
    void showText() {
        if (displayText)
            textView.setText( model.getResourceManager().getDataString() );
    }

    public 
    synchronized                    
    void showText( String text ) {
        if (displayText)
            textView.setText( text );
    }

    public
    synchronized                    
    void appendText( String text ) {
        if (displayText)
            textView.append( text );
    }

    public void keyPressed( KeyEvent e ) {
        // ESC
        if ( e.getKeyCode() == 27 ) {
            model.getResourceManager().stop();
        }
    }

    public void keyReleased( KeyEvent e ) {
    }

    public void keyTyped( KeyEvent e ) {
    }
    
}