package textanalyzer.ui.view;

import textanalyzer.ui.MainApplet;
import java.awt.Point;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Image;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Panel;

public abstract class View extends Panel implements Runnable, Active {
	
	protected MainApplet applet;
	private Activator activator;
	
	public ModelPoint point;
	public ModelPoint velocity;
	private double focusFactor;
	private double focusVelocity;
	public double scale;
    public boolean changed;
    
	public View( MainApplet applet ) {
        this.applet = applet;
        setBackground( applet.properties.UIBgColor );
		activator = new Activator( this );
		point = new ModelPoint( 0, 0 );
		point = new ModelPoint( 0, 0 );
		focusFactor = 500 / activator.refreshDelay;
		scale = 1;
		initFont();
	}
	
    public void run() {	activator.run(); }
	
	public void start() { activator.start(); }

	public void stop() { activator.stop(); }

	public synchronized void refresh() {
		repaint();
	}

	public int getRefreshRate() {
		return applet.properties.UIRefreshRate;
	}

	public void zoomInit() {
		scale = 1;
		initFont();
        changed = true;
        scale(scale);
	}

	public void zoom( double scaleFactor ) {
        if (scale > 10) return;
		scale *= scaleFactor;
		initFont();
        changed = true;
        scale(scale);
	}

    public abstract void scale(double sacale);
    
	protected Image offscreen;
    protected Graphics offgraphics;
	Font font;
	protected FontMetrics fm;
	//Dimension offscreensize;
    Dimension s;
	public Dimension size;

	public void initGraphics() {
		size = getSize();
		offscreen = createImage(size.width, size.height);
		offgraphics = offscreen.getGraphics();
		offgraphics.setFont( font );
		fm = offgraphics.getFontMetrics();
	}
	
	public void initFont() {
		int h = (int)( 8*scale );
		if ( h == 0 ) h = 1;
		//font = new Font("Arial",Font.PLAIN,(int)(12*scale) );
		font = new Font("Tahoma",Font.BOLD, h );
		if ( offgraphics != null ) {
            offgraphics.setFont( font );
			fm = offgraphics.getFontMetrics();
		}
	}
	
	Color colorBackground = new Color( 245, 245, 255 );
	Color colorBase = Color.gray;
	Color colorBaseLight = Color.lightGray;
	Color colorBaseDark = Color.darkGray;

	public synchronized void update( Graphics g ) {
		s = getSize();
		if ( offgraphics == null || s.width != size.width || s.height != size.height ) 
			initGraphics();
		offgraphics.setColor( colorBackground );
		offgraphics.fillRect( 0, 0, s.width, s.height );
		offgraphics.setColor( colorBaseDark );
		offgraphics.drawLine( 0, 0, s.width, 0        );
		offgraphics.drawLine( 0, 0, 0,       s.height );
		offgraphics.setColor( Color.white );
		offgraphics.drawLine( 1, s.height-1, s.width-1, s.height-1 );
		offgraphics.drawLine( s.width-1, 0, s.width-1, s.height-1 );
		offgraphics.clipRect( 1, 1, s.width - 2, s.height - 2 );
	}

	public Point to( ModelPoint mp ) {
		Point p = new Point();
		p.x = (int)( size.width/2 + scale*(mp.x - this.point.x) );
		p.y = (int)( size.height/2 + scale*(mp.y - this.point.y) );
		return p;
	}
	
	public ModelPoint from( Point p ) {
		ModelPoint mp = new ModelPoint();
		mp.x = this.point.x + (p.x - size.width/2)/scale;
		mp.y = this.point.y + (p.y - size.height/2)/scale;
		return mp;
	}
	
	public void moveTowards( ModelPoint p ) {
		double dx = p.x - point.x;
		double dy = p.y - point.y;
		double dl = Math.sqrt( dx*dx + dy*dy );
		focusVelocity = dl / focusFactor;
		double k = focusVelocity / dl;
		if ( k < 1 ) {
			point.x += k*dx;
			point.y += k*dy;	
		} else {
			point.x = p.x;
			point.y = p.y;	
		}
	}

}
