package textanalyzer.ui.view.graph;

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import textanalyzer.ui.view.ModelPoint;

public class GraphController 
             implements MouseListener, 
                        MouseMotionListener {
	
	private GraphModel model;
	private GraphView view;
	private Point pickpoint;
	
    public GraphController( GraphModel model, GraphView view ) {
		this.model = model;
		this.view = view;
		pickpoint = new Point();
    }

    public boolean inside( Node n, Point p ) {
        Point np = view.to( n.point );
        return ( ( Math.abs( np.x - p.x ) < n.w / 2 ) &&
                 ( Math.abs( np.y - p.y ) < n.h / 2 ) );
    }	


    // JDK1.1 event handling
	
    public void mouseClicked(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
		view.addMouseMotionListener(this);
		Point point = e.getPoint();
		Enumeration list = model.listNodes();
		boolean leftButton = ( (e.getModifiers() & e.BUTTON1_MASK) == e.BUTTON1_MASK );
		boolean rightButton = ( (e.getModifiers() & e.BUTTON3_MASK) == e.BUTTON3_MASK );
		boolean ctrlKey = e.isControlDown();
		boolean shiftKey = e.isShiftDown();
		boolean dblClick = (e.getClickCount() == 2);
		while ( list.hasMoreElements() ) {
			Node node = (Node)list.nextElement();
			if ( node.visible && inside( node, point ) ) {
				if ( leftButton ) {
					model.picked = node;
					Point nodepoint = view.to(node.point);
					pickpoint.x = e.getX() - nodepoint.x;
					pickpoint.y = e.getY() - nodepoint.y;
				}
				if ( dblClick && leftButton ) {
					model.root = node;
                    view.zoomInit();
                    view.zoom( 1 / node.weight );
					view.repaint();
					e.consume();
					return;
				}
				if ( ctrlKey && (leftButton || rightButton) ) {
					model.toggleSelected(node);
					view.repaint();
					e.consume();
					return;
				}
				if ( shiftKey && leftButton ) {
					model.addSelectedRange(node);
					view.repaint();
					e.consume();
					return;
				}
				if ( rightButton && model.hasSelected( node ) ) {
					model.dominant = node;
					view.repaint();
					e.consume();
					return;
				}
				model.clearSelected();
				model.addSelected(node);
				view.repaint();
				e.consume();
				return;
			}
		}
		model.clearSelected();
		e.consume();
    }

    public void mouseReleased(MouseEvent e) {
		boolean leftButton = ( (e.getModifiers() & e.BUTTON1_MASK) == e.BUTTON1_MASK );
		boolean rightButton = ( (e.getModifiers() & e.BUTTON3_MASK) == e.BUTTON3_MASK );
		if ( rightButton ) { 
			new ContextMenu( view, model, e.getX(), e.getY() );
		}
		if ( leftButton ) { 
			model.picked = null;
		}
		view.removeMouseMotionListener(this);
		view.repaint();
		e.consume();
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseDragged(MouseEvent e) {
		if( model.picked != null ) {
			Point newpoint = new Point();
			newpoint.x = e.getX() - pickpoint.x;
			newpoint.y = e.getY() - pickpoint.y;
			model.picked.point = view.from( newpoint );
			view.repaint();
		}
		e.consume();
	}

    public void mouseMoved(MouseEvent e) {
    }
	
}
