package textanalyzer.ui.view.graph;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import textanalyzer.ui.view.*;
import textanalyzer.model.*;

public
class      GraphModel 
implements AnalyzerUI {

    private Hashtable nodes;
    private Hashtable edges;
	
	public  Node   root;
	private Vector selected;
	public  Node   dominant;
	public  Node   picked;
    private ModelPoint domain;

	private ModelDynamics dynamics;
	private ModelLayout layout;
    private AnalyzerModel aModel;
	
	public GraphModel() {
		nodes = new Hashtable(50);
		edges = new Hashtable(50);
		selected = new Vector();
		dynamics = new NetDynamics( this );
		layout = new RandomLayout( this );
        domain = new ModelPoint(0,0);
	}
	
    public void setModel(AnalyzerModel aModel) {
        this.aModel = aModel;
    }
	
    public AnalyzerModel getModel() {
        return aModel;
    }

    // Nodes and Edges interface
    
    private 
    Node
    getNode( String name ) {
        Node node = (Node)nodes.get( name );
        if ( node == null ) {
            // create new Node
            node = new Node( name );
            nodes.put( name, node );
            layout.place( domain, node );
            domain = node.point;
            if ( root == null ) root = node;
        }
        return node;
    }

    private 
    Edge
    getEdge( String from, String to ) {
        Edge edge = (Edge)edges.get( from + to );
        if ( edge == null ) {
            edge = new Edge( getNode(from), getNode(to) );
            edges.put( from + to, edge );
        }
        return edge;
    }

    public Enumeration listNodes() {
		return nodes.elements();
	}
	
	public Enumeration listEdges() {
		return edges.elements();
	}

    void removeNode( String name ) {
        Node node = (Node)nodes.get( name );
        if ( node != null ) {
            Enumeration list = listEdges();
		    Edge e;
		    while( list.hasMoreElements() ) {
			    e = (Edge)list.nextElement();
                if ( e.from == node || e.to == node )
	    			edges.remove( e.from.name + e.to.name );
		    }
            nodes.remove(name);
        }
        aModel.markStopWord(name); // mark as StopWord in WordSystem
    }
    
    // Dynamics and Layout
    
	public void refresh() {
		dynamics.refresh();
	}

	public void setLayout( ModelLayout layout ) {
		this.layout = layout;
	}

    // Node selection implementation
    
	public void addSelected( Node node ) {
		dominant = node;
		selected.addElement( node );
	}
	
	public void toggleSelected( Node node ) {
		if ( selected.contains( node ) ) removeSelected( node );
		else addSelected( node );
	}
	
	public void addSelectedRange( Node node ) {
	}

	public void removeSelected( Node node ) {
		if ( node == dominant ) dominant = null;
		selected.removeElement( node );
	}
	
	public Enumeration listSelected() {
		return selected.elements();
	}
	
	public boolean hasSelected( Node node ) {
		Enumeration list = selected.elements();
		while ( list.hasMoreElements() ) {
			if ( node == (Node)list.nextElement() )
				return true;
		}
		return false;
	}

	public boolean isSelectedEmpty() {
		return selected.isEmpty();
	}
	
	public void clearSelected() {
		dominant = null;
		selected.removeAllElements();
	}

    
    // AnalyzerUI implementation
    
    public 
    void updateEntity( String name, double weight ) {
        getNode(name).setWeight( weight );
    }
    
    public 
    void updateRelation( String from, String to, double weight ) {
        getEdge( from, to ).setWeight( weight );        
    }    
    
    public
    void clear() {
        nodes.clear();
        edges.clear();
        root = null;
        dominant = null;
        clearSelected();
    }

}
