package textanalyzer.ui.view.graph;

import java.awt.*;

public class GraphPainter {
	
	protected GraphModel model;
	protected GraphView view;
	
    private Font fonts[];
    private static int 
        MIN_SIZE = 4,
        DEF_SIZE = 60,
        MAX_SIZE = 250;
    
	GraphPainter( GraphModel model, GraphView view ) {
		this.model = model;
		this.view = view;
        fonts = new Font[MAX_SIZE];
        for ( int i=MIN_SIZE; i<MAX_SIZE; i++ ) {
            fonts[i] = new Font( "Arial", Font.PLAIN, i );
        }
	}

	final Color 
		colorBorder = new Color( 150, 150, 150),
		colorPickedBorder = Color.red,
		colorRootBorder = Color.red,
		colorDominantBorder = Color.black,
		colorNode = new Color( 245, 245, 255 ),
		colorSelectedNode = Color.white,
		colorText = Color.black,
		colorTextBackground = new Color( 245, 245, 225 ),
		colorTextBorder = new Color( 150, 150, 150),
		colorEdge = new Color( 140, 140, 140);

	private Point p;
    private int w, h, asc;
    private Color border, base;
	public void paintNode( Graphics g, Node node ) {
		
        // calculate size
        if ( node.changed || view.changed ) {
            h = (int)( DEF_SIZE * node.weight * view.scale );
            if ( h < MIN_SIZE || h > MAX_SIZE ) {
                node.visible = false;
                return;
            }
            FontMetrics fm = g.getFontMetrics(fonts[h]);
		    w = fm.stringWidth( node.name ) + h/2 ;
            asc = fm.getAscent();
            node.h = h;
            node.w = w;
            node.asc = asc;
            node.changed = false;
        } else {
            h = node.h;
            w = node.w;
            asc = node.asc;
        }

        // calculate point
		p = view.to(node.point);
		if  ( p.x + w/2 > 0 && p.x-w/2 < view.size.width && 
			  p.y + h/2 > 0 && p.y-h/2 < view.size.height ) {
			node.visible = true; 
        } else {
			node.visible = false;
            return;
        }
        
        // choose Color
		if ( node == model.picked ) {
			border = colorPickedBorder;
		} else {
			if ( node == model.dominant ) {
				border = colorDominantBorder;
			} else {
				border = colorBorder;
			}
		}
		if ( model.hasSelected( node ) ) {
			base = colorSelectedNode;
		} else {
			base = colorNode;
		}

        g.setColor( base );
		g.fillRect( p.x - w/2, p.y - h/2, w, h );
		g.setColor( border );

		g.drawRect( p.x - w/2, p.y - h/2, w, h );
		g.setColor( colorText );

        // choose Font
        g.setFont( fonts[h] );
		
        // draw label
        g.drawString( node.name, p.x - w/2 + h/3, (p.y - h/2) + asc );
        
        // choose a career, choose a job ...

	}
	
	
	public void paintEdge( Graphics g, Edge e ) {
        if ( !e.from.visible && !e.to.visible ) return;
		Point from = view.to(e.from.point);
		Point to = view.to(e.to.point);
		//int length = (int)Math.abs(Math.sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2)) - e.length);
		g.setColor( colorEdge );
		g.drawLine( from.x, from.y, to.x, to.y );
	}

}
