package textanalyzer.ui.view.graph;

import java.util.Enumeration;

public class NetDynamics extends ModelDynamics {
	
	public NetDynamics(GraphModel model) {
		super(model);
	}
	
	double vx, vy, f, dx, dy, length, dlength;
	double m = 1.8;  // node mass
	double k = 0.05; // (0-1) proportion between node and edge interaction
	Enumeration edges, nodes1, nodes2;
	Edge e;
	Node n1,n2;
	
	public void refresh() {
		edges = model.listEdges();
		while ( edges.hasMoreElements() ) {
			e = (Edge)edges.nextElement();
			vx = e.to.point.x - e.from.point.x;
			vy = e.to.point.y - e.from.point.y;
			length = Math.sqrt(vx * vx + vy * vy);
            length = (length == 0) ? .001 : length;
			f = (e.length - length) / length / m * k;
			dx = f * vx;
			dy = f * vy;

			e.to.velocity.x += dx;
			e.to.velocity.y += dy;
			e.from.velocity.x -= dx;
			e.from.velocity.y -= dy;
		}
		nodes1 = model.listNodes();
		while ( nodes1.hasMoreElements() ) {
			n1 = (Node)nodes1.nextElement();
			dx = 0;
			dy = 0;
			nodes2 = model.listNodes();
			while ( nodes2.hasMoreElements() ) {
				n2 = (Node)nodes2.nextElement();
				if ( n1 == n2 ) continue;
				vx = n1.point.x - n2.point.x;
				vy = n1.point.y - n2.point.y;
				length = vx * vx + vy * vy;
				if (length == 0) {
					dx += -5 + 10 * Math.random();
					dy += -5 + 10 * Math.random();
				} else if (length < 10000) {
					dx += vx / length ;
					dy += vy / length ;
				}
			}
			dlength = dx * dx + dy * dy ;
			if (dlength > 0) {
				dlength = Math.sqrt(dlength) * m / (1-k); 
				// k added for force tuning
 				n1.velocity.x += dx / dlength;
				n1.velocity.y += dy / dlength;
			}
		}
		nodes1 = model.listNodes();
		while ( nodes1.hasMoreElements() ) {
			n1 = (Node)nodes1.nextElement();
            if ( n1 == model.picked || n1 == model.root ) continue;
            n1.point.x += Math.max(-5, Math.min(5, n1.velocity.x));
            n1.point.y += Math.max(-5, Math.min(5, n1.velocity.y));
			n1.velocity.x *= 0.5;
			n1.velocity.y *= 0.5;
		}
	}

}
